;;;;;;;;;;;;;;;;;;;;;;;;;;; -*- Mode: Emacs-Lisp -*- ;;;;;;;;;;;;;;;;;;;;;;;;;;
;; pvs-load.el -- Loads all the relevant PVS emacs files, and invokes the
;;                PVS image.  Displays the PVS Welcome buffer when done.
;; Author          : Sam Owre
;; Created On      : Fri Dec 17 13:32:31 1993
;; Last Modified By: Sam Owre
;; Last Modified On: Tue Nov 21 17:34:14 1995
;; Update Count    : 60
;; Status          : Beta test
;; 
;; HISTORY
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

;;; Define this first, so we can start logging right away.

(defun pvs-log-message (kind msg)
  (save-excursion
    (set-buffer (get-buffer-create "PVS Log"))
    (goto-char (point-max))
    (insert (format "%s(%s): %s\n"
		kind
	      (substring (current-time-string) 4 19)
	      msg))))

(defun pvs-msg (msg &rest args)
  (let ((m (apply 'format msg args)))
    (cond (noninteractive
	   (princ m)
	   (terpri))
	  (t
	   (pvs-log-message 'MSG m)
	   (message "%s" m)))))

(pvs-log-message 'LOG "Started loading Emacs files")

(defvar pvs-path nil)
(defvar pvs-image nil)

;;; Try to determine the version of Emacs being run.

(defvar emacs-system
  (cond ((string-match "XEmacs" (emacs-version))
	 'xemacs)
	((string-match "Lucid" (emacs-version))
	 'lemacs)
	((string-match "Emacs 20" (emacs-version))
	 'emacs20)
	((string-match "Emacs 19" (emacs-version))
	 'emacs19)
	((string-match "Emacs 18" (emacs-version))
	 'emacs18)))

;; The PVSPATH and PVSIMAGE environment variables must be set before this
;; file is loaded.

(if (getenv "PVSPATH")
    (setq pvs-path (if (string-match "/$" (getenv "PVSPATH"))
			(substring (getenv "PVSPATH") 0 -1)
			(getenv "PVSPATH")))
    (error "PVSPATH environment variable must be set"))

(if (getenv "PVSIMAGE")
    (setq pvs-image (getenv "PVSIMAGE"))
    (error "PVSIMAGE environment variable must be set"))

(defvar pvs-verbose
  (condition-case ()
      (car (read-from-string (getenv "PVSVERBOSE")))
    (error 0)))

(defvar pvs-validating nil)

(setq load-path
      (append (list pvs-path
		    (concat pvs-path
			    (if (memq emacs-system
				      '(emacs19 emacs20 lemacs xemacs))
				"/emacs/ilisp19"
				"/emacs/ilisp18")))
	      load-path
	      (list (concat pvs-path "/emacs"))))

(defvar *pvs-current-directory* default-directory
  "Pathname of the current PVS context.")

(require 'cl)

;; Use load here, rather than require, since we want to make sure we load
;; the version of comint we have control over (there are a large number of
;; comint.el files around, and they are largely incompatible).

(load "comint" nil noninteractive)
(setq comint-log t)
(load "popper" nil noninteractive)

(setq popper-pop-buffers ;; (nconc (list "PVS Error") popper-pop-buffers)
      t)
(setq popper-buffers-to-skip (list "PVS Error" " *Completions*"))
(setq lisp-no-popper nil)
(setq popper-use-message-buffer nil)

;;; This function is not defined in all Emacs

(if (not (fboundp 'full-copy-sparse-keymap))
    (defun full-copy-sparse-keymap (km)
      "Recursively copy the sparse keymap KM."
      (cond ((consp km)
	     (cons (full-copy-sparse-keymap (car km))
		   (full-copy-sparse-keymap (cdr km))))
	    (t km))))

(if (not (fboundp 'save-match-data))
    (defmacro save-match-data (&rest body)
  "Execute the BODY forms, restoring the global value of the match data."
  (let ((original (make-symbol "match-data")))
    (list 'let (list (list original '(match-data)))
	  (list 'unwind-protect
		(cons 'progn body)
		(list 'store-match-data original))))))

(if (not (fboundp 'comint-mem))
    (fset 'comint-mem 'member))

(defmacro defpvs (name class arglist docstring &rest body)
  "(defpvs NAME CLASS ARGLIST DOCSTRING BODY...):
define NAME as a PVS command."
  (list 'progn
	(list 'put (list 'quote name) ''pvs-command (list 'quote class))
	(cons 'defun (cons name (cons arglist (cons docstring body))))))

(require 'ilisp)

(load "pvs-ilisp" nil noninteractive)
(load "pvs-ilisp-mods" nil noninteractive)
(load "pvs-mode" nil noninteractive)
(load "pvs-view" nil noninteractive)
(load "pvs-file-list" nil noninteractive)
(load "pvs-browser" nil noninteractive)
(load "pvs-utils" nil noninteractive)
(load "pvs-cmds" nil noninteractive)
(load "pvs-print" nil noninteractive)
(load "pvs-prover" nil noninteractive)
(load "pvs-abbreviations" nil noninteractive)
(if (or (and (eq emacs-system 'xemacs)
	     (boundp 'emacs-major-version)
	     (or (= emacs-major-version 20)
		 (and (= emacs-major-version 19)
		      (>= emacs-minor-version 12))))
	(and (eq emacs-system 'emacs19)
	     (boundp 'emacs-major-version)
	     (>= emacs-minor-version 29))
	(eq emacs-system 'emacs20))
    (load "pvs-menu" nil noninteractive))
(load "tcl" nil noninteractive)
(load "pvs-tcl" nil noninteractive)
(load "comment" nil noninteractive)
(load "pvs-prover-helps" nil noninteractive)

(defpvs report-pvs-bug help ()
  "Sets up mail buffer for reporting PVS bugs."
  (interactive)
  (mail)
  (mail-to) (insert "pvs-bugs@csl.sri.com")
  (mail-subject))


(defun pvs-welcome (&optional display)
  (let ((cbuf (current-buffer))
	(buf (get-buffer-create "PVS Welcome"))
	(cdir *pvs-current-directory*)
	(vers (get-pvs-version-information)))
    (set-buffer buf)
    (setq fill-column (window-width))
    (if buffer-read-only (toggle-read-only))    
    (erase-buffer)
    (insert "\n\nSRI International\nComputer Science Laboratory")
    (insert "\n\n
            +----------------------------------------+
            |                                        |
            |  PPPPPPPP    VVV     VVV    SSSSSSSS   |
	    |  PPPPPPPPP   VVV     VVV   SSSSSSSSSS  |
	    |  PPP    PPP  VVV     VVV   SSS     SS  |
	    |  PPP    PPP  VVV     VVV   SSS         |
	    |  PPP    PPP  VVV     VVV   SSS         |
	    |  PPP    PPP  VVV     VVV   SSS         |
	    |  PPPPPPPPP   VVV     VVV   SSSSSSSSS   |
	    |  PPPPPPPP    VVV     VVV    SSSSSSSSS  |
	    |  PPP         VVV     VVV          SSS  |
	    |  PPP         VVV     VVV          SSS  |
	    |  PPP          VVV   VVV           SSS  |
	    |  PPP           VVV VVV     SS     SSS  |
	    |  PPP            VVVVV      SSSSSSSSSS  |
	    |  PPP             VVV        SSSSSSSS   |
            |                                        |
            +----------------------------------------+")
    (insert "\n\nWelcome to the PVS\nSpecification and Verification System")
    (insert "\n\nType C-c h for a summary of the commands.")
    (insert "\n\nYour current working context is\n" cdir)
    (insert "\n\nUse M-x change-context to move to a different context.")
    (insert "\n\n-----------------------------------------------------------------")
    (insert "\n\n" (pvs-version-string))
    (insert "\n\nPLEASE CHECK THE FOLLOWING FILE PERIODICALLY FOR NEWS OF LATER VERSIONS")
    (insert "\nhttp://www.csl.sri.com/pvs/versions or\n ftp://www.csl.sri.com/pub/pvs/versions")
    (insert "\n\n" (cadr (cdddr vers)) "\n" (cadr (cddddr vers)))
    (if (string-match "lucid" pvs-image)
	(insert-string "
   This version of PVS also requires a Lucid Lisp License.\n"))
    (insert-string "
   ----------
   Bug reports and suggestions for improvement should be sent to
   pvs-bugs@csl.sri.com
   ----------
   Questions may be sent to pvs-help@csl.sri.com; for details send
   a message to pvs-help-request@csl.sri.com with Subject: help
   ----------
   If you wish to get on the PVS mailing list, send a request to
   pvs-request@csl.sri.com")
    (insert "\n\n-----------------------------------------------------------------")
    (insert "\n
   PVS (Prototype Verification System) is  made freely available  by
   SRI for  experimental  use under the  terms   of the PVS  License
   Agreement.  Note in particular that PVS is provided ``as is'' and
   with  absolutely no warranty  of  any   kind.  PVS may be further
   redistributed under the terms of its license agreement.")

    (condition-case ()
	(center-region (point-min) (point))
      (error nil))
    (set-buffer-modified-p nil)
    (pvs-mode)				; change mode of welcome buffer
    (cd cdir)
    (toggle-read-only)
    (goto-char (point-min))
    (cond (display
	   (switch-to-buffer buf)
	   ;;(delete-other-windows)
	   )
	  (t (set-buffer cbuf)))
    buf))

(defpvs pvs environment ()
  "Starts the PVS process

The pvs command starts the pvs lisp process.  This is useful if the Lisp
process has died for some reason, and you wish to keep using the same
Emacs session.  Note that you will still need to retypecheck your files to
get to the same state."
  (interactive)
  (setq debug-on-error t)
  (when current-prefix-arg
    (let ((num (prefix-numeric-value current-prefix-arg)))
      (if (and (<= 0 num) (<= num 3))
	  (setenv "PVSPATCHLEVEL" num)
	  (message "Illegal patchlevel number - %s" num)))
    (setq current-prefix-arg nil))
  (unless noninteractive
    (message "Initializing PVS: please wait..."))
  ;;(switch-to-buffer "pvs")
  (save-excursion
    (set-buffer (get-buffer-create "PVS Log"))
    (pvs-view-mode))
  (save-excursion
    (setq *pvs-initialized* nil)
    (pvs-init)
    ;;(setq debug-on-quit t)
    (while (and (not (equal (simple-status-pvs) "Done"))
		(equal (process-status (ilisp-process)) 'run))
      (accept-process-output (ilisp-process))))
  (setq debug-on-quit nil)
  (unless (equal (process-status (ilisp-process)) 'run)
    (switch-to-buffer "*pvs*")
    (error "Could not run PVS"))
  (save-excursion
    (set-buffer (get-buffer "*pvs*"))
    (set-syntax-table pvs-mode-syntax-table))
  ;;(set-package-lisp "PVS")
  (pvs-send-and-wait "(progn (in-package \"PVS\") nil)" nil nil 'dont-care)
  (sleep-for 1)
  (if (< (pvs-major-version-number) 2)
      (load "patch" t t)
      (load (format "patch%d" (pvs-major-version-number)) t t))
  (let ((comint-log nil))
    (pvs-send-and-wait
     (format "(progn (setq *pvs-path* \"%s\")
                     (pvs-init)
                     (setq *noninteractive* %s)
                     (setq *pvs-verbose* %d))"
	 pvs-path noninteractive pvs-verbose)
     nil nil 'dont-care))
  (setq *pvs-version-information* nil)
  (sleep-for 1)
  ;;(check-screen-width)
  ;; sets *pvs-current-directory* and pops up the welcome buffer
  (init-change-context *pvs-current-directory*)
  (setq pvs-in-checker nil)
  (unless noninteractive
    (pvs-welcome))
  (when (file-exists-p "~/.pvsemacs")
    (load "~/.pvsemacs"))
  (run-hooks 'change-context-hook)
  (if (pvs-buffer-file-name)
      (pvs-mode)
      (unless noninteractive
	(switch-to-buffer (get-buffer-create "PVS Welcome"))))
  (setq debug-on-error nil)
  (unless noninteractive
    (message "Ready")))

(pvs)
